#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2009-2012 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

from . import read_bin_file, SkoolKitError

def get_snapshot(fname):
    ext = fname[-4:]
    if ext not in ('.sna', '.z80'):
        raise SnapshotError("{0}: Unknown file type '{1}'".format(fname, ext[1:]))
    data = read_bin_file(fname)
    if ext == '.sna':
        ram = data[27:]
    else:
        version = 1 if sum(data[6:8]) > 0 else 2
        header_size = 30 if version == 1 else 32 + data[30]
        header = data[:header_size]
        if version == 1 and header[12] & 32 == 0:
            ram = data[header_size:-4]
        else:
            ram = _decompress(data[header_size:], version)
    if len(ram) != 49152:
        raise SnapshotError("RAM size is {0}".format(len(ram)))
    mem = [0] * 16384
    mem.extend(ram)
    return mem

def _decompress(ramz, version):
    if version == 1:
        return _decompress_block(ramz[:-4])
    blocks = {}
    j = 0
    while j < len(ramz):
        length = ramz[j] + 256 * ramz[j + 1]
        if length == 65535:
            blocks[ramz[j + 2]] = ramz[j + 3:j + 16387]
            j += 16387
        else:
            blocks[ramz[j + 2]] = _decompress_block(ramz[j + 3:j + 3 + length])
            j += 3 + length
    return blocks[8] + blocks[4] + blocks[5]

def _decompress_block(ramz):
    block = []
    i = 0
    while i < len(ramz):
        b = ramz[i]
        i += 1
        if b == 237:
            c = ramz[i]
            i += 1
            if c == 237:
                length, byte = ramz[i], ramz[i + 1]
                if length == 0:
                    raise SnapshotError("Found ED ED 00 {0:02X}".format(byte))
                block += [byte] * length
                i += 2
            else:
                block += [b, c]
        else:
            block.append(b)
    return block

class SnapshotError(SkoolKitError):
    pass
